@@ -35,7 +35,8 @@ class TestCase(unittest.TestCase):
 
         b = Node()
         n.appendChild(b)
-        self.assertEqual(True, n.hasChildNodes)
+        # note this was fixed from being a property to a method in 6.6.7
+        self.assertEqual(True, n.hasChildNodes())
 
         c = Node()
         n.appendChild(c)
@@ -1088,7 +1089,6 @@ class TestCase(unittest.TestCase):
         # not able to recreate. Comment was updated to a Node in 6.1
         # this may have been due to that
 
-
     def test_treewalker(self):
 
         from domonic.dom import Comment
